/* --COPYRIGHT--,BSD
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  uart.c - c file for UART communication using MSP430 TimerA
 *         - peripheral
 *
 ******************************************************************************/
#include "uart.h"

unsigned int txData;

extern void receivedDataCommand(unsigned char receiveData);

/*******************************************************************************
 *
 *! This function initializes the necessary configurations or GPIO for UART
 *! functionality. It automatically selects Timer_UART or hardware UART using
 *! USCI_A0.
 *!
 *! \param data character data to be transmitted via UART.
 *!
 *! \return None
 *
 ******************************************************************************/
void uartInit(void)
{
    /* P1.2 = RXD, P1.3=TXD */
    P1SEL0 |= UART_TXD | UART_RXD ;

    /* Put eUSCI in reset */
    UCA0CTLW0 |= UCSWRST;

    /* SMCLK */
    UCA0CTLW0 |= UCSSEL__SMCLK;             // Put eUSCI in reset

    /* 12MHz 9600 */
    UCA0BR0 = 78;                           // 12000000/16/9600
    UCA0BR1 = 0x00;
    UCA0MCTLW = 0x1000 | UCOS16 | 0x0020;

    /* **Initialize USCI state machine** */
    UCA0CTLW0 &= ~UCSWRST;

    /* Enable USCI_A0 RX interrupt */
    UCA0IE |= UCRXIE;;
}

/*******************************************************************************
 *
 *! This function transmits a character via UART. It automatically selects
 *! Timer_UART or hardware UART using USCI_A0.
 *!
 *! \param data character data to be transmitted via UART.
 *!
 *! \return None
 *
 ******************************************************************************/
void uartTxByte(unsigned char byte)
{
    /* eUSCI_A0 TX buffer ready? */
	while(!(UCA0IFG & UCTXIFG));
    /* transmite data */
    UCA0TXBUF = byte;
}

/*******************************************************************************
 *
 *! Hardware UART interrupt service routine
 *!
 *! \param None
 *!
 *! \return None
 *
 ******************************************************************************/
void eUSCIA0IsrHandler(void)
{
    /* eUSCI_A0 TX buffer ready? */
	while(!(UCA0IFG & UCTXIFG));

    /* Parse received command immediately */
    receivedDataCommand(UCA0RXBUF);
}
